/**
   httpGetTask.cpp
*/
/*
  (c)2023 Forward Computing and Control Pty. Ltd.
  NSW Australia, www.forward.com.au
  This code is not warranted to be fit for any purpose. You may only use it at your own risk.
  This code may be freely used for both private and commercial use
  Provided this copyright is maintained.
*/

#include <Arduino.h>
#include "volatileVars.h"
#include <HTTPClient.h>
#include "millisDelay.h"
#include "SafeString.h"
#include "httpWindowStateTask.h"
#include "locks.h"

extern void processWindowOpenLine(SafeString & sfName);

static char host_path[256];

//#define DEBUG
static Print* debugPtr = NULL;

#define MAX_RESPONSE_SIZE 1024

static volatile unsigned long badResponseCount = 0;
static const unsigned long MAX_BAD_RESPONSE_COUNT = 10; // allow 10 failures => 30sec

bool isHttpWindowStateConnectionGood() {
  return (badResponseCount < MAX_BAD_RESPONSE_COUNT);
}

void setHttpWindowStateTaskDebugOut(Print * out) {
  debugPtr = out;
}


void parseResponse(SafeString &sfRespInit) {
  cSF(sfLine, 120);
  int idx = 0;
  while (idx >= 0) {
    idx = sfRespInit.stoken(sfLine, idx, "\r\n", false);
    sfLine.trim();
    processWindowOpenLine(sfLine);
  }
}

static char httpURL[256];
static void getHttpTask(void* parameter) { // parameter not used

  while (true) {
    if (wifiConnected_v) {
      HTTPClient http;
      http.begin(host_path);
      // start connection and send HTTP header
      int httpCode = http.GET();
      // httpCode will be negative on error
      if (httpCode > 0) {
        if (httpCode == HTTP_CODE_OK) {
          badResponseCount = 0;
          cSF(sfResponse, MAX_RESPONSE_SIZE);
          String payload = http.getString();
          //          if (debugPtr) {
          //            debugPtr->println(payload);
          //          }
          sfResponse.readFrom(payload.c_str());
          parseResponse(sfResponse);
        } else {
          badResponseCount = badResponseCount + 1;
        }
      } else {
        badResponseCount = badResponseCount + 1;
        if (debugPtr) {
          debugPtr->printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
        }
      }
      http.end();
      delay(3000);
    }
  } // while(1)
}

bool createHttpWindowStateTask(const char* _host_path) {
  cSFA(sfHostPath, host_path);
  sfHostPath = _host_path;

  BaseType_t err = xTaskCreatePinnedToCore(
                     getHttpTask,          // Task function.
                     "getHttpTask",        // name of task.
                     4 * 1024,           // Stack size of task
                     NULL,                     // parameter of the task
                     2,                        // priority of the task  // loop() has priority == 1, 2 is higher but has delay in it
                     NULL,                    // Task handle to keep track of created task
                     0);
  if (((int)err) != 1) { // pPass ,  pdTRUE      ( ( BaseType_t ) 1 ) in #include "freertos/projdefs.h"
    if (debugPtr) {
      debugPtr->print("xTaskCreatePinnedToCore returned:");
      debugPtr->println((int)err);
    }
    return false;
  }
  // could not create
  return true;
}
